#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdio.h>

#include "uart0.h"

#define XTAL    8000000L
#define DEBOUNCE  256L
#define uchar unsigned char
#define uint unsigned int
uchar prescaler;

// Uhrzeit
uint8_t hour = 0;
uint8_t minute = 0;
uint8_t sec = 0;
uint volatile second;

// TIMER 1
volatile uint8_t tick_sec;

ISR(TIMER1_COMPA_vect) {
#if XTAL % DEBOUNCE
  OCR1A = XTAL / DEBOUNCE - 1;
#endif
  if( --prescaler == 0 ) {
    prescaler = (uchar)DEBOUNCE;
    second++;
#if XTAL % DEBOUNCE
    OCR1A = XTAL / DEBOUNCE + XTAL % DEBOUNCE - 1;
#endif
  }

}

void tick_time() {
	sec++;
	if (sec > 59) {
		sec = 0;
		minute++;
		if (minute > 59) {
			minute = 0;
			hour++;
			if (hour > 23) {
				hour = 0;
			}
		}
	}
}

int main (void) {

	uart0_init();

	TCCR1B = (1<<WGM12) | (1<<CS10);
	OCR1A = XTAL / DEBOUNCE - 1;
	TCNT1 = 0;
	prescaler = (uchar)DEBOUNCE;
	second = 0;

	TIMSK1 = (1<<OCIE1A);

	sei();

	while(1) {
		if (second > 0) {
			second = 0;

			tick_time();
			sprintf(uart0_buf,"Time: %02i:%02i:%02i\r\n",hour,minute,sec);
			uart0_putstr((unsigned int*)uart0_buf);
		}
	}
}
